#!/usr/bin/env python
import testsupport
import os, string, sys, types, unittest, weakref
import sqlite

class DBAPICompliance(unittest.TestCase):
    def CheckAPILevel(self):
        self.assertEqual(sqlite.apilevel, '2.0',
                         'apilevel is %s, should be 2.0' % sqlite.apilevel)

    def CheckThreadSafety(self):
        self.assertEqual(sqlite.threadsafety, 1,
                         'threadsafety is %d, should be 1' % sqlite.threadsafety)

    def CheckParamStyle(self):
        self.assertEqual(sqlite.paramstyle, 'pyformat',
                         'paramstyle is "%s", should be "pyformat"' %
                         sqlite.paramstyle)

    def CheckWarning(self):
        self.assert_(issubclass(sqlite.Warning, StandardError),
                     'Warning is not a subclass of StandardError')

    def CheckError(self):
        self.failUnless(issubclass(sqlite.Error, StandardError),
                        'Error is not a subclass of StandardError')

    def CheckInterfaceError(self):
        self.failUnless(issubclass(sqlite.InterfaceError, sqlite.Error),
                        'InterfaceError is not a subclass of Error')

    def CheckDatabaseError(self):
        self.failUnless(issubclass(sqlite.DatabaseError, sqlite.Error),
                        'DatabaseError is not a subclass of Error')

    def CheckDataError(self):
        self.failUnless(issubclass(sqlite.DataError, sqlite.DatabaseError),
                        'DataError is not a subclass of DatabaseError')

    def CheckOperationalError(self):
        self.failUnless(issubclass(sqlite.OperationalError, sqlite.DatabaseError),
                        'OperationalError is not a subclass of DatabaseError')

    def CheckIntegrityError(self):
        self.failUnless(issubclass(sqlite.IntegrityError, sqlite.DatabaseError),
                        'IntegrityError is not a subclass of DatabaseError')

    def CheckInternalError(self):
        self.failUnless(issubclass(sqlite.InternalError, sqlite.DatabaseError),
                        'InternalError is not a subclass of DatabaseError')

    def CheckProgrammingError(self):
        self.failUnless(issubclass(sqlite.ProgrammingError, sqlite.DatabaseError),
                        'ProgrammingError is not a subclass of DatabaseError')

    def CheckNotSupportedError(self):
        self.failUnless(issubclass(sqlite.NotSupportedError,
                                   sqlite.DatabaseError),
                        'NotSupportedError is not a subclass of DatabaseError')

class moduleTestCases(unittest.TestCase, testsupport.TestSupport):
    def setUp(self):
        self.filename = self.getfilename()
        self.cnx = sqlite.connect(self.filename)
        self.cur = self.cnx.cursor()

    def tearDown(self):
        try:
            self.cnx.close()
            self.removefile()
        except AttributeError:
            pass
        except sqlite.ProgrammingError:
            pass

    def CheckConnectionObject(self):
        self.assert_(isinstance(self.cnx, sqlite.Connection),
                     'sqlite.connect did not return a Connection object')

    def CheckConnectionClose(self):
        self.assert_(hasattr(self.cnx, 'close') and
                     type(self.cnx.close) == types.MethodType,
                     'close is not a method of Connection')
        self.cnx.close()
        self.removefile()
        self.failUnlessRaises(sqlite.ProgrammingError, self.cnx.close)

    def CheckConnectionCommit(self):
        self.assert_(hasattr(self.cnx, "commit") and
                     type(self.cnx.commit) == types.MethodType,
                     'commit is not a method of Connection')
        self.cnx.close()
        self.removefile()
        self.failUnlessRaises(sqlite.ProgrammingError, self.cnx.commit)

    def CheckConnectionRollback(self):
        self.assert_(hasattr(self.cnx, "rollback") and
                     type(self.cnx.rollback) == types.MethodType,
                     'rollback is not a method of Connection')
        self.cnx.close()
        self.removefile()
        self.failUnlessRaises(sqlite.ProgrammingError, self.cnx.rollback)

    def CheckConnectionCursor(self):
        self.assert_(hasattr(self.cnx, "cursor") and
                     type(self.cnx.cursor) == types.MethodType,
                     'cursor is not a method of Connection')
        self.cnx.close()
        self.removefile()
        self.failUnlessRaises(sqlite.ProgrammingError, self.cnx.cursor)

    def CheckCloseConnection(self):
        self.cnx.close()
        self.removefile()

    def CheckCursorObject(self):
        self.assert_(isinstance(self.cur, sqlite.Cursor),
                     'cnx.cursor() did not return a Cursor instance')

    def CheckCursorArraysize(self):
        self.assert_(self.cur.arraysize == 1,
                     'cur.arraysize is %d, it should be 1' %
                     self.cur.arraysize)

    def CheckCursorDescription(self):
        self.assert_(self.cur.description == None,
                     "cur.description should be None at this point, it isn't.")

    def CheckCursorRowcount(self):
        self.assert_(self.cur.rowcount == -1,
                     'cur.rowcount is %d, should be -1' % self.cur.rowcount)

    def CheckCursorClose(self):
        self.assert_(hasattr(self.cur, "close") and
                     type(self.cur.close) == types.MethodType,
                     'close is not a method of the Cursor object')
        self.cur.close()
        self.failUnlessRaises(sqlite.ProgrammingError, self.cur.close)

    def CheckCursorExecute(self):
        self.assert_(hasattr(self.cur, "execute") and
                     type(self.cur.execute) == types.MethodType,
                     'execute is not a method of the Cursor object')
        self.cur.close()
        self.failUnlessRaises(sqlite.ProgrammingError,
                              self.cur.execute, 'SELECT max(3,4)')

    def CheckCursorExecutemany(self):
        self.assert_(hasattr(self.cur, "executemany") and
                     type(self.cur.executemany) == types.MethodType,
                     'executemany is not a method of the Cursor object')

        self.cur.close()
        self.failUnlessRaises(sqlite.ProgrammingError,
                              self.cur.executemany, 'SELECT max(3,4)', [1,2])

    def CheckCursorFetchone(self):
        self.assert_(hasattr(self.cur, "fetchone") and
                     type(self.cur.fetchone) == types.MethodType,
                     'fetchone is not a method of the Cursor object')
        self.cur.close()
        self.failUnlessRaises(sqlite.ProgrammingError, self.cur.fetchone)

    def CheckCursorFetchMany(self):
        self.failUnless(hasattr(self.cur, "fetchmany") and
                        type(self.cur.fetchmany) == types.MethodType,
                        'fetchmany is not a method of the Cursor object')

        cursor = self.cnx.cursor()
        cursor.execute("create table test(id int)")
        cursor.executemany("insert into test(id) values (%s)", range(10))
        cursor.execute("select id from test")
        res = cursor.fetchmany()
        self.failUnlessEqual(len(res), 1, """fetchmany should have returned a
            list of length 1, but the list was %i elements long""" % len(res))
        res = cursor.fetchmany(2)
        self.failUnlessEqual(len(res), 2, """fetchmany should have returned a
            list of length 2, but the list was %i elements long""" % len(res))
        cursor.arraysize = 5
        res = cursor.fetchmany()
        self.failUnlessEqual(len(res), 5, """fetchmany should have returned a
            list of length 5, but the list was %i elements long""" % len(res))

        self.cur.close()
        self.failUnlessRaises(sqlite.ProgrammingError,
                              self.cur.fetchmany, 10)

    def CheckCursorFetchall(self):
        self.failUnless(hasattr(self.cur, "fetchall") and
                        type(self.cur.fetchall) == types.MethodType,
                        'fetchall is not a method of the Cursor object')
        self.cur.close()
        self.failUnlessRaises(sqlite.ProgrammingError,
                              self.cur.fetchall)

    def CheckCursorSetoutputsize(self):
        self.failUnless(hasattr(self.cur, "setoutputsize") and
                        type(self.cur.setoutputsize) == types.MethodType,
                        'setoutputsize is not a method of the Cursor object')
        self.cur.close()
        self.failUnlessRaises(sqlite.ProgrammingError,
                              self.cur.setoutputsize, 1024)

    def CheckCursorSetinputsizes(self):
        self.failUnless(hasattr(self.cur, "setinputsizes") and
                        type(self.cur.setinputsizes) == types.MethodType,
                        'setinputsizes is not a method of the Cursor object')
        self.cur.close()
        self.failUnlessRaises(sqlite.ProgrammingError,
                              self.cur.setinputsizes, [1, 2, 3])

    def CheckExecuteWithSingleton(self):
        """Test execute() with a singleton string as the parameter."""
        try:
            self.cur.execute("select max(3,4)")
        except StandardError, msg:
            self.fail(msg)

        self.assertEqual(type(self.cur.description), types.TupleType,
                         "cur.description should be a tuple, but isn't.")

        clen = len(self.cur.description)
        self.assertEqual(clen, 1,
                         "Length of cur.description is %d, it should be %d." %
                         (clen, 1))


        self.assertEqual(len(self.cur.description[0]), 7,
                         "Length of cur.description[0] is %d, it should be 7." %
                         len(self.cur.description[0]))


        self.failUnless(self.cur.description[0][0] == "max(3,4)"    and
                        self.cur.description[0][1] == sqlite.NUMBER and
                        self.cur.description[0][2] == None          and
                        self.cur.description[0][3] == None          and
                        self.cur.description[0][4] == None          and
                        self.cur.description[0][5] == None          and
                        self.cur.description[0][6] == None,
                        "cur.description[0] does not match the query.")
        self.cur.close()

    def CheckExecuteWithTuple(self):
        """Test execute() with a tuple as the parameter."""
        try:
            self.cur.execute("select max(%s, %s)", (4, 5))
        except StandardError, msg:
            self.fail(msg)

        # Empty tuple
        try:
            self.cur.execute("select 3+4", ())
        except StandardError, msg:
            self.fail(msg)
        self.cur.close()

    def CheckExecuteWithDictionary(self):
        """Test execute() with a dictionary as the parameter."""
        try:
            self.cur.execute("select max(%(n1)s, %(n2)s)", {"n1": 5, "n2": 6})
        except StandardError, msg:
            self.fail(msg)
        self.cur.close()

    def CheckQuotingOfLong(self):
        """Test wether longs are quoted properly for SQL."""
        try:
            self.cur.execute("-- types long")
            self.cur.execute("select %s + %s as x", (5L, 6L))
        except StandardError, msg:
            self.fail(msg)
        res = self.cur.fetchone()
        self.failUnlessEqual(res.x, 11L,
            "The addition of long should have returned %i, returned %i"
                % (11L, res.x))

    def CheckCursorIterator(self):
        self.cur.execute("create table test (id, name)")
        self.cur.executemany("insert into test (id) values (%s)",
                            [(1,), (2,), (3,)])
        self.cur.execute("-- types int")
        self.cur.execute("select id from test")

        if sys.version_info[:2] >= (2,2):
            counter = 0
            for row in self.cur:
                if counter == 0:
                    self.failUnlessEqual(row.id, 1,
                        "row.id should have been 1, was %i" % row.id)
                elif counter == 1:
                    self.failUnlessEqual(row.id, 2,
                        "row.id should have been 2, was %i" % row.id)
                elif counter == 2:
                    self.failUnlessEqual(row.id, 3,
                        "row.id should have been 3, was %i" % row.id)
                else:
                    self.fail("Iterated over too many rows.")
                counter += 1
        else:
            # Python 2.1
            counter = 0
            try:
                while 1:
                    row = self.cur.next()
                    if counter == 0:
                        self.failUnlessEqual(row.id, 1,
                            "row.id should have been 1, was %i" % row.id)
                    elif counter == 1:
                        self.failUnlessEqual(row.id, 2,
                            "row.id should have been 2, was %i" % row.id)
                    elif counter == 2:
                        self.failUnlessEqual(row.id, 3,
                            "row.id should have been 3, was %i" % row.id)
                    else:
                        self.fail("Iterated over too many rows.")
                    counter += 1
            except IndexError:
                pass
            self.failUnlessEqual(counter, 3,
                "Should have iterated over 3 items, was: %i" % counter)

    def CheckCursorScrollAndRownumber(self):
        self.cur.execute("create table test (id, name)")
        values = [("foo",)] * 20
        self.cur.executemany("insert into test (name) values (%s)", values)
        self.cur.execute("select name from test")
        self.failUnlessEqual(self.cur.rownumber, 0,
            "Directly after execute, rownumber must be 0, is: %i"
                % self.cur.rownumber)

        self.cur.scroll(1, "absolute")
        self.cur.scroll(5, "absolute")
        self.failUnlessEqual(self.cur.rownumber, 5,
            "rownumber should be 5, is: %i"
                % self.cur.rownumber)

        self.cur.scroll(1, "relative")
        self.failUnlessEqual(self.cur.rownumber, 6,
            "rownumber should be 6, is: %i"
                % self.cur.rownumber)

        self.cur.scroll(-2, "relative")
        self.failUnlessEqual(self.cur.rownumber, 4,
            "rownumber should be 4, is: %i"
                % self.cur.rownumber)

        self.failUnlessRaises(IndexError, self.cur.scroll, -2, "absolute")
        self.failUnlessRaises(IndexError, self.cur.scroll, 1000, "absolute")

        self.cur.scroll(10, "absolute")
        self.failUnlessRaises(IndexError, self.cur.scroll, -11, "relative")

        self.cur.scroll(10, "absolute")
        self.failUnlessRaises(IndexError, self.cur.scroll, 30, "relative")

    def CheckCursorConnection(self):
        if not isinstance(self.cur.connection, weakref.ProxyType) and \
           not isinstance(self.cur.connection, weakref.CallableProxyType):
            fail("cursor.connection doesn't return the correct type")

    def CheckCursorLastRowID(self):
        self.cur.execute("create table test (id integer primary key, name)")

        self.cur.execute("insert into test(name) values ('foo')")
        self.failUnlessEqual(self.cur.lastrowid, 1,
            "lastrowid should be 1, is %i" % self.cur.lastrowid)

        # Test MySQLdb compatibility function
        self.failUnlessEqual(self.cur.lastrowid, self.cnx.insert_id())

        self.cur.execute("insert into test(name) values ('foo')")
        self.failUnlessEqual(self.cur.lastrowid, 2,
            "lastrowid should be 2, is %i" % self.cur.lastrowid)

    def CheckResultObject(self):
        try:
            self.cur.execute("select max(3,4)")
            self.assertEqual(self.cur.rowcount, 1,
                             "cur.rowcount is %d, it should be 1." %
                             self.cur.rowcount)
            self.res = self.cur.fetchall()
        except StandardError, msg:
            self.fail(msg)

        self.assertEqual(type(self.res), types.ListType,
                         'cur.fetchall() did not return a sequence.')

        self.assertEqual(len(self.res), 1,
                         'Length of the list of results is %d, it should be 1' %
                         len(self.res))

        self.failUnless(isinstance(self.res[0], sqlite.PgResultSet),
                        'cur.fetchall() did not return a list of PgResultSets.')

    def CheckResultFetchone(self):
        try:
            self.cur.execute("select max(3,4)")
            self.res = self.cur.fetchone()
            self.assertEqual(self.cur.rowcount, 1,
                             'cur.rowcount is %d, it should be 1.' %
                             self.cur.rowcount)
        except StandardError, msg:
            self.fail(msg)

        self.failUnless(isinstance(self.res, sqlite.PgResultSet),
                        "cur.fetchone() does not return a PgResultSet.")

        try:
            self.res = self.cur.fetchone()
            self.assertEqual(self.res, None,
                             "res should be None at this point, but it isn't.")
        except StandardError, msg:
            self.fail(msg)

    def CheckRowCountAfterInsert(self):
        try:
            self.cur.execute("create table test(a)")
            self.cur.execute("insert into test(a) values (5)")
            self.assertEqual(self.cur.rowcount, 1,
                            'cur.rowcount is %d, it should be 1.' %
                            self.cur.rowcount)
        except StandardError, msg:
            self.fail(msg)
    
    def CheckRowCountAfterUpdate(self):
        try:
            self.cur.execute("create table test(a, b)")
            self.cur.execute("insert into test(a, b) values (1, 2)")
            self.cur.execute("insert into test(a, b) values (1, 3)")
            self.cur.execute("insert into test(a, b) values (1, 4)")
            self.cur.execute("update test set b=1 where a=1")
            self.assertEqual(self.cur.rowcount, 3,
                            'cur.rowcount is %d, it should be 3.' %
                            self.cur.rowcount)
        except StandardError, msg:
            self.fail(msg)

    def CheckRowCountAfterDelete(self):
        try:
            self.cur.execute("create table test(a, b)")
            self.cur.execute("insert into test(a, b) values (1, 2)")
            self.cur.execute("insert into test(a, b) values (1, 3)")
            self.cur.execute("insert into test(a, b) values (2, 4)")
            self.cur.execute("delete from test where a=1")
            self.assertEqual(self.cur.rowcount, 2,
                            'cur.rowcount is %d, it should be 2.' %
                            self.cur.rowcount)
        except StandardError, msg:
            self.fail(msg)

    def CheckSelectOfNonPrintableString(self):
        try:
            a = '\x01\x02\x03\x04'
            self.cur.execute('select %s as a', a)
            r = self.cur.fetchone()
            self.assertEqual(len(r.a), len(a),
                             "Length of result is %d, it should be %d."  %
                             (len(r.a), len(a)))
            self.failUnless(r.a == a,
                             "Result is '%s', it should be '%s'" % (r.a, a))
        except StandardError, msg:
            self.fail(msg)

    def CheckQuotingIntWithPercentS(self):
        try:
            self.cur.execute("create table test(a number)")
            self.cur.execute("insert into test(a) values (%s)", (5,))
        except StandardError, msg:
            self.fail(msg)

    def CheckQuotingLongWithPercentS(self):
        try:
            self.cur.execute("create table test(a number)")
            self.cur.execute("insert into test(a) values (%s)", (50000000L,))
        except StandardError, msg:
            self.fail(msg)

    def CheckQuotingFloatWithPercentS(self):
        try:
            self.cur.execute("create table test(a number)")
            self.cur.execute("insert into test(a) values (%s)", (-3.24,))
        except StandardError, msg:
            self.fail(msg)

    def CheckQuotingIntWithPyQuoting(self):
        try:
            self.cur.execute("create table test(a number)")
            self.cur.execute("insert into test(a) values (%i)", (5,))
        except StandardError, msg:
            self.fail(msg)

    def CheckQuotingLongWithPyQuoting(self):
        try:
            self.cur.execute("create table test(a number)")
            self.cur.execute("insert into test(a) values (%i)", (50000000L,))
        except StandardError, msg:
            self.fail(msg)

    def CheckQuotingFloatWithPyQuoting(self):
        try:
            self.cur.execute("create table test(a number)")
            self.cur.execute("insert into test(a) values (%f)", (-3.24,))
        except StandardError, msg:
            self.fail(msg)

def suite():
    dbapi_suite = unittest.makeSuite(DBAPICompliance, "Check")
    module_suite = unittest.makeSuite(moduleTestCases, "Check")
    test_suite = unittest.TestSuite((dbapi_suite, module_suite))
    return test_suite

def main():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
This combines all PySQLite test suites into one big one.
"""

import unittest, sys
import api_tests, logging_tests, lowlevel_tests, pgresultset_tests, type_tests
import userfunction_tests, transaction_tests

def suite():
    suite = unittest.TestSuite((lowlevel_tests.suite(), api_tests.suite(),
            type_tests.suite(), userfunction_tests.suite(),
            transaction_tests.suite(), pgresultset_tests.suite(),
            logging_tests.suite()))

    return suite

def main():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    main()



#!/usr/bin/env python
import testsupport
import os, unittest, sys
import sqlite

class PgResultSetTests(unittest.TestCase, testsupport.TestSupport):
    def setUp(self):
        self.filename = self.getfilename()
        self.cnx = sqlite.connect(self.filename)
        self.cur = self.cnx.cursor()

    def tearDown(self):
        try:
            self.cnx.close()
            self.removefile()
        except AttributeError:
            pass
        except sqlite.ProgrammingError:
            pass

    def getResult(self):
        try:
            self.cur.execute("DROP TABLE TEST")
        except sqlite.DatabaseError, reason:
            pass

        self.cur.execute("CREATE TABLE TEST (id, name, age)")
        self.cur.execute("INSERT INTO TEST (id, name, age) VALUES (%s, %s, %s)",
                            (5, 'Alice', 29))
        self.cur.execute("-- types int, str, int")
        self.cur.execute("SELECT id, name, age FROM TEST")
        return self.cur.fetchone()

    def CheckAttributeAccess(self):
        res = self.getResult()
        if not hasattr(res, "id"):
            self.fail("Resultset doesn't have attribute 'id'")
        if not hasattr(res, "ID"):
            self.fail("Resultset doesn't have attribute 'ID'")

    def CheckAttributeValue(self):
        res = self.getResult()
        if res.id != 5:
            self.fail("id should be 5, is %i" % res.id)
        if res.ID != 5:
            self.fail("ID should be 5, is %i" % res.ID)

    def CheckKeyAccess(self):
        res = self.getResult()
        if not "id" in res:
            self.fail("Resultset doesn't have item 'id'")
        if not "ID" in res:
            self.fail("Resultset doesn't have item 'ID'")

    def CheckKeyValue(self):
        res = self.getResult()
        if res["id"] != 5:
            self.fail("id should be 5, is %i" % res.id)
        if res["ID"] != 5:
            self.fail("ID should be 5, is %i" % res.ID)

    def CheckIndexValue(self):
        res = self.getResult()
        if res[0] != 5:
            self.fail("item 0 should be 5, is %i" % res.id)

    def Check_haskey(self):
        res = self.getResult()
        if not res.has_key("id"):
            self.fail("resultset should have key 'id'")
        if not res.has_key("ID"):
            self.fail("resultset should have key 'ID'")
        if not res.has_key("Id"):
            self.fail("resultset should have key 'Id'")

    def Check_len(self):
        l = len(self.getResult())
        if l != 3:
            self.fail("length of resultset should be 3, is %i", l)

    def Check_keys(self):
        res = self.getResult()
        if res.keys() != ["id", "name", "age"]:
            self.fail("keys() should return %s, returns %s" %
                        (["id", "name", "age"], res.keys()))

    def Check_values(self):
        val = self.getResult().values()
        if val != (5, 'Alice', 29):
            self.fail("Wrong values(): %s" % val)

    def Check_items(self):
        it = self.getResult().items()
        if it != [("id", 5), ("name", 'Alice'), ("age", 29)]:
            self.fail("Wrong items(): %s" % it)

    def Check_get(self):
        res = self.getResult()
        v = res.get("id")
        if v != 5:
            self.fail("Wrong result for get [1]")

        v = res.get("ID")
        if v != 5:
            self.fail("Wrong result for get [2]")

        v = res.get("asdf")
        if v is not None:
            self.fail("Wrong result for get [3]")

        v = res.get("asdf", 6)
        if v != 6:
            self.fail("Wrong result for get [4]")

class TupleResultTests(unittest.TestCase, testsupport.TestSupport):
    def setUp(self):
        self.filename = self.getfilename()
        self.cnx = sqlite.connect(self.filename)
        self.cnx.rowclass = tuple
        self.cur = self.cnx.cursor()

    def tearDown(self):
        try:
            self.cnx.close()
            self.removefile()
        except AttributeError:
            pass
        except sqlite.ProgrammingError:
            pass

    def getOneResult(self):
        try:
            self.cur.execute("DROP TABLE TEST")
        except sqlite.DatabaseError, reason:
            pass

        self.cur.execute("CREATE TABLE TEST (id, name, age)")
        self.cur.execute("INSERT INTO TEST (id, name, age) VALUES (%s, %s, %s)",
                            (5, 'Alice', 29))
        self.cur.execute("-- types int, str, int")
        self.cur.execute("SELECT id, name, age FROM TEST")
        return self.cur.fetchone()

    def getManyResults(self):
        try:
            self.cur.execute("DROP TABLE TEST")
        except sqlite.DatabaseError, reason:
            pass

        self.cur.execute("CREATE TABLE TEST (id, name, age)")
        self.cur.execute("INSERT INTO TEST (id, name, age) VALUES (%s, %s, %s)",
                            (5, 'Alice', 29))
        self.cur.execute("INSERT INTO TEST (id, name, age) VALUES (%s, %s, %s)",
                            (5, 'Alice', 29))
        self.cur.execute("INSERT INTO TEST (id, name, age) VALUES (%s, %s, %s)",
                            (5, 'Alice', 29))
        self.cur.execute("-- types int, str, int")
        self.cur.execute("SELECT id, name, age FROM TEST")
        return self.cur.fetchmany(2)

    def getAllResults(self):
        try:
            self.cur.execute("DROP TABLE TEST")
        except sqlite.DatabaseError, reason:
            pass

        self.cur.execute("CREATE TABLE TEST (id, name, age)")
        self.cur.execute("INSERT INTO TEST (id, name, age) VALUES (%s, %s, %s)",
                            (5, 'Alice', 29))
        self.cur.execute("INSERT INTO TEST (id, name, age) VALUES (%s, %s, %s)",
                            (5, 'Alice', 29))
        self.cur.execute("INSERT INTO TEST (id, name, age) VALUES (%s, %s, %s)",
                            (5, 'Alice', 29))
        self.cur.execute("-- types int, str, int")
        self.cur.execute("SELECT id, name, age FROM TEST")
        return self.cur.fetchall()

    def CheckRowTypeIsTupleFetchone(self):
        res = self.getOneResult()
        self.failUnless(type(res) is tuple, "Result type of row isn't a tuple")

    def CheckRowTypeIsTupleFetchmany(self):
        res = self.getManyResults()
        self.failUnless(type(res[1]) is tuple, "Result type of row isn't a tuple")

    def CheckRowTypeIsTupleFetchall(self):
        res = self.getAllResults()
        self.failUnless(type(res[2]) is tuple, "Result type of row isn't a tuple")

def suite():
    tests = [unittest.makeSuite(PgResultSetTests, "Check"),
                                unittest.makeSuite(PgResultSetTests, "Check")]
    if sys.version_info >= (2,2):
        tests.append(unittest.makeSuite(TupleResultTests, "Check"))
    return unittest.TestSuite(tests)

def main():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    main()



#!/usr/bin/env python
import testsupport
import os, string, sys, types, unittest
import sqlite

def intreturner(x):
    return int(x) * 2

def floatreturner(x):
    return float(x) * 2.0

def stringreturner(x):
    return "[%s]" % x

def nullreturner(x):
    return None

def exceptionreturner(x):
    return 5 / 0

class MySum:
    def __init__(self):
        self.reset()

    def reset(self):
        self.sum = 0

    def step(self, x):
        self.sum += int(x)

    def finalize(self):
        val = self.sum
        self.reset()
        return val

class MySumFloat:
    def __init__(self):
        self.reset()

    def reset(self):
        self.sum = 0.0

    def step(self, x):
        self.sum += float(x)

    def finalize(self):
        val = self.sum
        self.reset()
        return val

class MySumReturnNull:
    def __init__(self):
        self.reset()

    def reset(self):
        self.sum = 0

    def step(self, x):
        self.sum += int(x)

    def finalize(self):
        return None

class MySumStepExeption:
    def __init__(self):
        self.reset()

    def reset(self):
        self.sum = 0

    def step(self, x):
        self.sum += int(x) / 0

    def finalize(self):
        val = self.sum
        self.reset()
        return val

class MySumFinalizeExeption:
    def __init__(self):
        self.reset()

    def reset(self):
        self.sum = 0

    def step(self, x):
        self.sum += int(x)

    def finalize(self):
        val = self.sum / 0
        self.reset()
        return val

class UserFunctions(unittest.TestCase, testsupport.TestSupport):
    def setUp(self):
        self.filename = self.getfilename()
        self.cnx = sqlite.connect(self.filename)

        sqlite._sqlite.enable_callback_debugging(0)

        self.cnx.create_function("intreturner", 1, intreturner)
        self.cnx.create_function("floatreturner", 1, floatreturner)
        self.cnx.create_function("stringreturner", 1, stringreturner)
        self.cnx.create_function("nullreturner", 1, nullreturner)
        self.cnx.create_function("exceptionreturner", 1, exceptionreturner)

        self.cnx.create_aggregate("mysum", 1, MySum)
        self.cnx.create_aggregate("mysumfloat", 1, MySumFloat)
        self.cnx.create_aggregate("mysumreturnnull", 1, MySumReturnNull )
        self.cnx.create_aggregate("mysumstepexception", 1, MySumStepExeption)
        self.cnx.create_aggregate("mysumfinalizeexception", 1, MySumFinalizeExeption)
        self.cur = self.cnx.cursor()

    def tearDown(self):
        try:
            self.cnx.close()
            self.removefile()
        except AttributeError:
            pass
        except sqlite.InterfaceError:
            pass

    def CheckIntFunction(self):
        self.cur.execute("create table test (a)")
        self.cur.execute("insert into test(a) values (%s)", 5)
        self.cur.execute("-- types int")
        self.cur.execute("select intreturner(a) as a from test")
        res = self.cur.fetchone()
        self.failUnless(isinstance(res.a, types.IntType),
                        "The result should have been an int.")
        self.failUnlessEqual(res.a, 10,
                        "The function returned the wrong result.")

    def CheckFloatFunction(self):
        self.cur.execute("create table test (a)")
        self.cur.execute("insert into test(a) values (%s)", 5.0)
        self.cur.execute("-- types float")
        self.cur.execute("select floatreturner(a) as a from test")
        res = self.cur.fetchone()
        self.failUnless(isinstance(res.a, types.FloatType),
                        "The result should have been a float.")
        self.failUnlessEqual(res.a, 5.0 * 2.0,
                        "The function returned the wrong result.")

    def CheckStringFunction(self):
        mystr = "test"
        self.cur.execute("create table test (a)")
        self.cur.execute("insert into test(a) values (%s)", mystr)
        self.cur.execute("-- types str")
        self.cur.execute("select stringreturner(a) as a from test")
        res = self.cur.fetchone()
        self.failUnless(isinstance(res.a, types.StringType),
                        "The result should have been a string.")
        self.failUnlessEqual(res.a, "[%s]" % mystr,
                        "The function returned the wrong result.")

    def CheckNullFunction(self):
        mystr = "test"
        self.cur.execute("create table test (a)")
        self.cur.execute("insert into test(a) values (%s)", mystr)
        self.cur.execute("-- types str")
        self.cur.execute("select nullreturner(a) as a from test")
        res = self.cur.fetchone()
        self.failUnlessEqual(res.a, None,
                        "The result should have been None.")

    def CheckFunctionWithNullArgument(self):
        mystr = "test"
        self.cur.execute("-- types str")
        self.cur.execute("select nullreturner(NULL) as a")
        res = self.cur.fetchone()
        self.failUnlessEqual(res.a, None,
                        "The result should have been None.")


    def CheckExceptionFunction(self):
        mystr = "test"
        self.cur.execute("create table test (a)")
        self.cur.execute("insert into test(a) values (%s)", mystr)
        self.cur.execute("-- types str")
        try:
            self.cur.execute("select exceptionreturner(a) as a from test")
        except sqlite.DatabaseError, reason:
            pass
        except Exception, reason:
            self.fail("Wrong exception raised: %s", sys.exc_info()[0])

    def CheckAggregateBasic(self):
        self.cur.execute("create table test (a)")
        self.cur.executemany("insert into test(a) values (%s)", [(10,), (20,), (30,)])
        self.cur.execute("-- types int")
        self.cur.execute("select mysum(a) as sum from test")
        res = self.cur.fetchone()
        self.failUnless(isinstance(res.sum, types.IntType),
                        "The result should have been an int.")
        self.failUnlessEqual(res.sum, 60,
                        "The function returned the wrong result.")

    def CheckAggregateFloat(self):
        self.cur.execute("create table test (a)")
        self.cur.executemany("insert into test(a) values (%s)", [(10.0,), (20.0,), (30.0,)])
        self.cur.execute("-- types float")
        self.cur.execute("select mysumfloat(a) as sum from test")
        res = self.cur.fetchone()
        self.failUnless(isinstance(res.sum, types.FloatType),
                        "The result should have been an float.")
        if res.sum <= 59.9 or res.sum >= 60.1:
            self.fail("The function returned the wrong result.")

    def CheckAggregateReturnNull(self):
        self.cur.execute("create table test (a)")
        self.cur.executemany("insert into test(a) values (%s)", [(10,), (20,), (30,)])
        self.cur.execute("-- types int")
        self.cur.execute("select mysumreturnnull(a) as sum from test")
        res = self.cur.fetchone()
        self.failUnlessEqual(res.sum, None,
                        "The result should have been None.")

    def CheckAggregateStepException(self):
        self.cur.execute("create table test (a)")
        self.cur.executemany("insert into test(a) values (%s)", [(10,), (20,), (30,)])
        self.cur.execute("-- types int")
        try:
            self.cur.execute("select mysumstepexception(a) as sum from test")
        except sqlite.DatabaseError, reason:
            pass
        except Exception, reason:
            self.fail("Wrong exception raised: %s" % sys.exc_info()[0])

    def CheckAggregateFinalizeException(self):
        self.cur.execute("create table test (a)")
        self.cur.executemany("insert into test(a) values (%s)", [(10,), (20,), (30,)])
        self.cur.execute("-- types int")
        try:
            self.cur.execute("select mysumfinalizeexception(a) as sum from test")
        except sqlite.DatabaseError, reason:
            pass
        except Exception, reason:
            self.fail("Wrong exception raised: %s", sys.exc_info()[0])

    def CheckAggregateStepNullArgument(self):
        self.cur.execute("-- types int")
        self.cur.execute("select mysum(NULL) as a")
        res = self.cur.fetchone()
        self.failUnlessEqual(res.a, 0,
                        "The result should have been 0.")


def suite():
    user_functions = unittest.makeSuite(UserFunctions, "Check")
    test_suite = unittest.TestSuite((user_functions,))
    return test_suite

def main():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    main()



import _sqlite

"""Python interface to the SQLite embedded database engine."""

#-------------------------------------------------------------------------------
# Module Information
#-------------------------------------------------------------------------------

__revision__ = """$Revision: 1.22 $"""[11:-2]

threadsafety = 1
apilevel = "2.0"
paramstyle = "pyformat"

# This is the version string for the current PySQLite version.
version = "1.0.1"

# This is a tuple with the same digits as the vesrion string, but it's
# suitable for comparisons of various versions.
version_info = (1, 0, 1)

#-------------------------------------------------------------------------------
# Data type support
#-------------------------------------------------------------------------------

from main import DBAPITypeObject, Cursor, Connection, PgResultSet

STRING    = DBAPITypeObject(_sqlite.STRING)

BINARY    = DBAPITypeObject(_sqlite.BINARY)

INT       = DBAPITypeObject(_sqlite.INTEGER)

NUMBER    = DBAPITypeObject(_sqlite.INTEGER,
                            _sqlite.FLOAT)

DATE      = DBAPITypeObject(_sqlite.DATE)

TIME      = DBAPITypeObject(_sqlite.TIME)

TIMESTAMP = DBAPITypeObject(_sqlite.TIMESTAMP)

ROWID     = DBAPITypeObject()

# Nonstandard extension:
UNICODESTRING = DBAPITypeObject(_sqlite.UNICODESTRING)

#-------------------------------------------------------------------------------
# Exceptions
#-------------------------------------------------------------------------------

from _sqlite import Warning, Error, InterfaceError, \
    DatabaseError, DataError, OperationalError, IntegrityError, InternalError, \
    ProgrammingError, NotSupportedError

#-------------------------------------------------------------------------------
# Global Functions
#-------------------------------------------------------------------------------

def connect(*args, **kwargs):
    return Connection(*args, **kwargs)

from _sqlite import encode, decode

Binary = encode

__all__ = ['connect','IntegrityError', 'InterfaceError', 'InternalError',
           'NotSupportedError', 'OperationalError',
           'ProgrammingError', 'Warning',
           'Connection', 'Cursor', 'PgResultSet',
           'apilevel', 'paramstyle', 'threadsafety', 'version', 'version_info',
           'Binary', 'decode']



from __future__ import nested_scopes
import _sqlite

import copy, new, sys, weakref
from types import *

try:
    from mx import DateTime
    have_datetime = 1
except ImportError:
    have_datetime = 0

if have_datetime:
    # Make the required Date/Time constructor visable in the PySQLite module.
    Date = DateTime.Date
    Time = DateTime.Time
    Timestamp = DateTime.Timestamp
    DateFromTicks = DateTime.DateFromTicks
    TimeFromTicks = DateTime.TimeFromTicks
    TimestampFromTicks = DateTime.TimestampFromTicks

    # And also the DateTime types
    DateTimeType = DateTime.DateTimeType
    DateTimeDeltaType = DateTime.DateTimeDeltaType

class DBAPITypeObject:
    def __init__(self,*values):
        self.values = values

    def __cmp__(self,other):
        if other in self.values:
            return 0
        if other < self.values:
            return 1
        else:
            return -1

def _quote(value):
    """_quote(value) -> string

    This function transforms the Python value into a string suitable to send to
    the SQLite database in a SQL statement.  This function is automatically
    applied to all parameters sent with an execute() call.  Because of this a
    SQL statement string in an execute() call should only use '%s' [or
    '%(name)s'] for variable substitution without any quoting."""

    if value is None:
        return 'NULL'
    elif type(value) in (IntType, LongType, FloatType):
        return value
    elif isinstance(value, StringType):
        return "'%s'" % value.replace("'", "''")
    elif hasattr(value, '__quote__'):
        return value.__quote__()
    elif hasattr(value, '_quote'):
        return value._quote()
    elif have_datetime and type(value) in \
            (DateTime.DateTimeType, DateTime.DateTimeDeltaType):
        return "'%s'" % value
    else:
        return repr(value)

def _quoteall(vdict):
    """_quoteall(vdict)->dict
    Quotes all elements in a list or dictionary to make them suitable for
    insertion in a SQL statement."""

    if type(vdict) is DictType or isinstance(vdict, PgResultSet):
        t = {}
        for k, v in vdict.items():
            t[k]=_quote(v)
    elif isinstance(vdict, StringType) or isinstance(vdict, UnicodeType):
        # Note: a string is a SequenceType, but is treated as a single
        #    entity, not a sequence of characters.
        t = (_quote(vdict), )
    elif type(vdict)in (ListType, TupleType):
        t = tuple(map(_quote, vdict))
    else:
        raise TypeError, \
              "argument to _quoteall must be a sequence or dictionary!"

    return t

class PgResultSet:
    """A DB-API query result set for a single row.
    This class emulates a sequence with the added feature of being able to
    reference a column as attribute or with dictionary access in addition to a
    zero-based numeric index."""

    def __init__(self, value):
        self.__dict__['baseObj'] = value

    def __getattr__(self, key):
        key = key.upper()
        if self._xlatkey.has_key(key):
            return self.baseObj[self._xlatkey[key]]
        raise AttributeError, key

    def __len__(self):
        return len(self.baseObj)

    def __getitem__(self, key):
        if isinstance(key, StringType):
            key = self.__class__._xlatkey[key.upper()]
        return self.baseObj[key]

    def __contains__(self, key):
        return self.has_key(key)

    def __getslice__(self, i, j):
        klass = make_PgResultSetClass(self._desc_[i:j])
        obj = klass(self.baseObj[i:j])
        return obj

    def __repr__(self):
        return repr(self.baseObj)

    def __str__(self):
        return str(self.baseObj)

    def __cmp__(self, other):
        return cmp(self.baseObj, other)

    def description(self):
        return self._desc_

    def keys(self):
        _k = []
        for _i in self._desc_:
            _k.append(_i[0])
        return _k

    def values(self):
        return self.baseObj[:]

    def items(self):
        _items = []
        for i in range(len(self.baseObj)):
            _items.append((self._desc_[i][0], self.baseObj[i]))

        return _items

    def has_key(self, key):
        return self._xlatkey.has_key(key.upper())

    def get(self, key, defaultval=None):
        if self.has_key(key):
            return self[key]
        else:
            return defaultval

def make_PgResultSetClass(description):
    NewClass = new.classobj("PgResultSetConcreteClass", (PgResultSet,), {})
    NewClass.__dict__['_desc_'] = description

    NewClass.__dict__['_xlatkey'] = {}

    for _i in range(len(description)):
        NewClass.__dict__['_xlatkey'][description[_i][0].upper()] = _i

    return NewClass

class Cursor:
    """Abstract cursor class implementing what all cursor classes have in
    common."""

    def __init__(self, conn, rowclass=PgResultSet):
        self.arraysize = 1

        # Add ourselves to the list of cursors for our owning connection.
        self.con = weakref.proxy(conn)
        self.con.cursors[id(self)] = self

        self.rowclass = rowclass

        self._reset()
        self.current_recnum = -1

    def _reset(self):
        # closed is a trinary variable:
        #     == None => Cursor has not been opened.
        #     ==    0 => Cursor is open.
        #     ==    1 => Cursor is closed.
        self.closed = None
        self.rowcount = -1
        self._real_rowcount = 0
        self.description = None
        self.rs = None
        self.current_recnum = 0

    def _checkNotClosed(self, methodname=None):
        if self.closed:
            raise _sqlite.ProgrammingError, \
                "%s failed - the cursor is closed." % (methodname or "")

    def _unicodeConvert(self, obj):
        """Encode all unicode strings that can be found in obj into
        byte-strings using the encoding specified in the connection's
        constructor, available here as self.con.encoding."""

        if isinstance(obj, StringType):
            return obj
        elif isinstance(obj, UnicodeType):
            return obj.encode(*self.con.encoding)
        elif isinstance(obj, ListType) or isinstance(obj, TupleType):
            converted_obj = []
            for item in obj:
                if isinstance(item, UnicodeType):
                    converted_obj.append(item.encode(*self.con.encoding))
                else:
                    converted_obj.append(item)
            return converted_obj
        elif isinstance(obj, DictType):
            converted_obj = {}
            for k, v in obj.items():
                if isinstance(v, UnicodeType):
                    converted_obj[k] = v.encode(*self.con.encoding)
                else:
                    converted_obj[k] = v
            return converted_obj
        elif isinstance(obj, PgResultSet):
            obj = copy.copy(obj)
            for k, v in obj.items():
                if isinstance(v, UnicodeType):
                    obj[k] = v.encode(*self.con.encoding)
            return obj
        else:
            return obj

    def execute(self, SQL, *parms):
        self._checkNotClosed("execute")

        if self.con.autocommit:
            pass
        else:
            if not(self.con.inTransaction or SQL[:6].upper() in ("SELECT","VACUUM","DETACH")):
                self.con._begin()
                self.con.inTransaction = 1

        SQL = self._unicodeConvert(SQL)

        if len(parms) == 0:
            # If there are no paramters, just execute the query.
            self.rs = self.con.db.execute(SQL)
        else:
            if len(parms) == 1 and \
               (type(parms[0]) in (DictType, ListType, TupleType) or \
                        isinstance(parms[0], PgResultSet)):
                parms = (self._unicodeConvert(parms[0]),)
                parms = _quoteall(parms[0])
            else:
                parms = self._unicodeConvert(parms)
                parms = tuple(map(_quote, parms))

            self.rs = self.con.db.execute(SQL % parms)

        self.closed = 0
        self.current_recnum = 0

        self.rowcount, self._real_rowcount = [len(self.rs.row_list)] * 2
        if self.rowcount == 0:
            self.rowcount = self.con.db.sqlite_changes()

        self.description = self.rs.col_defs

        if issubclass(self.rowclass, PgResultSet):
            self.rowclass = make_PgResultSetClass(self.description[:])

    def executemany(self, query, parm_sequence):
        self._checkNotClosed("executemany")

        if self.con is None:
            raise _sqlite.ProgrammingError, "connection is closed."

        for _i in parm_sequence:
            self.execute(query, _i)

    def close(self):
        if self.con and self.con.closed:
            raise _sqlite.ProgrammingError, \
                  "This cursor's connection is already closed."
        if self.closed:
            raise _sqlite.ProgrammingError, \
                  "This cursor is already closed."
        self.closed = 1

        # Disassociate ourselves from our connection.
        try:
            cursors = self.con.cursors
            del cursors.data[id(self)]
        except:
            pass

    def __del__(self):
        # Disassociate ourselves from our connection.
        try:
            cursors = self.con.cursors
            del cursors.data[id(self)]
        except:
            pass

    def setinputsizes(self, sizes):
        """Does nothing, required by DB API."""
        self._checkNotClosed("setinputsize")

    def setoutputsize(self, size, column=None):
        """Does nothing, required by DB API."""
        self._checkNotClosed("setinputsize")

    #
    # DB-API methods:
    #

    def fetchone(self):
        self._checkNotClosed("fetchone")

        # If there are no records
        if self._real_rowcount == 0:
            return None

        # If we have reached the last record
        if self.current_recnum >= self._real_rowcount:
            return None

        if type(self.rowclass) is TupleType:
            retval = self.rs.row_list[self.current_recnum]
        else:
            retval = self.rowclass(self.rs.row_list[self.current_recnum])
        self.current_recnum += 1

        return retval

    def fetchmany(self, howmany=None):
        self._checkNotClosed("fetchmany")

        if howmany is None:
            howmany = self.arraysize

        # If there are no records
        if self._real_rowcount == 0:
            return []

        # If we have reached the last record
        if self.current_recnum >= self._real_rowcount:
            return []

        if type(self.rowclass) is TupleType:
            retval = self.rs.row_list[self.current_recnum:self.current_recnum + howmany]
        else:
            retval = [self.rowclass(row) for row in self.rs.row_list[self.current_recnum:self.current_recnum + howmany]]

        self.current_recnum += howmany
        if self.current_recnum > self._real_rowcount:
            self.current_recnum = self._real_rowcount

        return retval

    def fetchall(self):
        self._checkNotClosed("fetchall")

        # If there are no records
        if self._real_rowcount == 0:
            return []

        # If we have reached the last record
        if self.current_recnum >= self._real_rowcount:
            return []

        if type(self.rowclass) is TupleType:
            retval = self.rs.row_list[self.current_recnum:]
        else:
            retval = [self.rowclass(row) for row in self.rs.row_list[self.current_recnum:]]

        self.current_recnum =self._real_rowcount

        return retval

    #
    # Optional DB-API extensions from PEP 0249:
    #

    def __iter__(self):
        return self

    def next(self):
        item = self.fetchone()
        if item is None:
            if sys.version_info[:2] >= (2,2):
                raise StopIteration
            else:
                raise IndexError
        else:
            return item

    def scroll(self, value, mode="relative"):
        if mode == "relative":
            new_recnum = self.current_recnum + value
        elif mode == "absolute":
            new_recnum = value
        else:
            raise ValueError, "invalid mode parameter"
        if new_recnum >= 0 and new_recnum < self.rowcount:
            self.current_recnum = new_recnum
        else:
            raise IndexError

    def __getattr__(self, key):
        if self.__dict__.has_key(key):
            return self.__dict__[key]
        elif key == "sql":
            # The sql attribute is a PySQLite extension.
            return self.con.db.sql
        elif key == "rownumber":
            return self.current_recnum
        elif key == "lastrowid":
            return self.con.db.sqlite_last_insert_rowid()
        elif key == "connection":
            return self.con
        else:
            raise AttributeError, key

class UnicodeConverter:
    def __init__(self, encoding):
        self.encoding = encoding

    def __call__(self, val):
        return unicode(val, *self.encoding)

class Connection:

    def __init__(self, database=None, mode=0755, converters={}, autocommit=0, encoding=None, timeout=None, command_logfile=None, *arg, **kwargs):
        # Old parameter names, for backwards compatibility
        database = database or kwargs.get("db")
        encoding = encoding or kwargs.get("client_encoding")

        # Set these here, to prevent an attribute access error in __del__
        # in case the connect fails.
        self.closed = 0
        self.db = None
        self.inTransaction = 0
        self.autocommit = autocommit
        self.cursors = weakref.WeakValueDictionary()
        self.rowclass = PgResultSet

        self.db = _sqlite.connect(database, mode)

        if type(encoding) not in (TupleType, ListType):
            self.encoding = (encoding or sys.getdefaultencoding(),)
        else:
            self.encoding = encoding

        register = self.db.register_converter
        # These are the converters we provide by default ...
        register("str", str)
        register("int", int)
        register("long", long)
        register("float", float)
        register("unicode", UnicodeConverter(self.encoding))
        register("binary", _sqlite.decode)

        # ... and DateTime/DateTimeDelta, if we have the mx.DateTime module.
        if have_datetime:
            register("date", DateTime.DateFrom)
            register("time", DateTime.TimeFrom)
            register("timestamp", DateTime.DateTimeFrom)
            register("interval", DateTime.DateTimeDeltaFrom)

        for typename, conv in converters.items():
            register(typename, conv)

        if timeout is not None:
            self.db.sqlite_busy_timeout(timeout)

        self.db.set_command_logfile(command_logfile)

    def __del__(self):
        if not self.closed:
            self.close()

    def _checkNotClosed(self, methodname):
        if self.closed:
            raise _sqlite.ProgrammingError, \
                  "%s failed - Connection is closed." % methodname

    def __anyCursorsLeft(self):
        return len(self.cursors.data.keys()) > 0

    def __closeCursors(self, doclose=0):
        """__closeCursors() - closes all cursors associated with this connection"""
        if self.__anyCursorsLeft():
            cursors = map(lambda x: x(), self.cursors.data.values())

            for cursor in cursors:
                try:
                    if doclose:
                        cursor.close()
                    else:
                        cursor._reset()
                except weakref.ReferenceError:
                    pass

    def _begin(self):
        self.db.execute("BEGIN")
        self.inTransaction = 1

    #
    # PySQLite extensions:
    #

    def create_function(self, name, nargs, func):
        self.db.create_function(name, nargs, func)

    def create_aggregate(self, name, nargs, agg_class):
        self.db.create_aggregate(name, nargs, agg_class)

    #
    # DB-API methods:
    #

    def commit(self):
        self._checkNotClosed("commit")
        if self.autocommit:
            # Ignore .commit(), according to the DB-API spec.
            return

        if self.inTransaction:
            self.db.execute("COMMIT")
            self.inTransaction = 0

    def rollback(self):
        self._checkNotClosed("rollback")
        if self.autocommit:
            raise _sqlite.ProgrammingError, "Rollback failed - autocommit is on."

        if self.inTransaction:
            self.db.execute("ROLLBACK")
            self.inTransaction = 0

    def close(self):
        self._checkNotClosed("close")

        self.__closeCursors(1)

        if self.inTransaction:
            self.rollback()

        if self.db:
            self.db.close()
        self.closed = 1

    def cursor(self):
        self._checkNotClosed("cursor")
        return Cursor(self, self.rowclass)

    #
    # Optional DB-API extensions from PEP 0249:
    #

    def __getattr__(self, key):
        if key in self.__dict__.keys():
            return self.__dict__[key]
        elif key in ('IntegrityError', 'InterfaceError', 'InternalError',
                     'NotSupportedError', 'OperationalError',
                     'ProgrammingError', 'Warning'):
            return getattr(_sqlite, key)
        else:
            raise AttributeError, key

    #
    # MySQLdb compatibility stuff
    #

    def insert_id(self):
        return self.db.sqlite_last_insert_rowid()



#!/usr/bin/env python
# This is a test case I got from a user that will crash PySQLite 0.5.0.
# It stress-tests PySQLite in multithreaded mode.

import os
import sys
import threading
import time
import random
import sqlite

dbname = "test.db"

#MECHANISM = "no timeout"
#MECHANISM = "use timeout"
#MECHANISM = "use slow busy handler"
MECHANISM = "use fast busy handler"

class Modifier(threading.Thread):
    def __init__(self, dbname):
        threading.Thread.__init__(self)
        self.dbname = dbname
    def run(self):
        print "Modifier: start"
        cx = sqlite.connect(self.dbname)
        cu = cx.cursor()
        print "Modifier: INSERTing"
        cu.execute("INSERT INTO meta (name, value) VALUES (%s, %s)",
                   "foo", "blah blah blah")
        for i in range(5):
            print "Modifier: sleeping %d" % i
            time.sleep(1)
        print "Modifier: committing"
        cx.commit()
        print "Modifier: committed"
        cu.close()
        cx.close()
        print "Modifier: end"

class Reader(threading.Thread):
    def __init__(self, name, dbname):
        threading.Thread.__init__(self, name=name)
        self.dbname = dbname
    def busyHandler(self, delay, table, numAttempts):
        print "Reader %s: busyHandler(delay=%r, table=%r, numAttempts=%r)"\
              % (self.getName(), delay, table, numAttempts)
        time.sleep(delay)
        return 1
    def run(self):
        print "Reader %s: start" % self.getName()
        if MECHANISM == "no timeout":
            cx = sqlite.connect(self.dbname)
        elif MECHANISM == "use timeout":
            cx = sqlite.connect(self.dbname, timeout=5000)
        elif MECHANISM == "use slow busy handler":
            cx = sqlite.connect(self.dbname)
            cx.db.sqlite_busy_handler(self.busyHandler, 1.0)
        elif MECHANISM == "use fast busy handler":
            cx = sqlite.connect(self.dbname, Xtimeout=5000.0)
            cx.db.sqlite_busy_handler(self.busyHandler, 0.1)
        else:
            raise ValueError("MECHANISM is not one of the expected values")
        sleepFor = random.randint(0, 3)
        print "Reader %s: sleeping for %d seconds" % (self.getName(), sleepFor)
        time.sleep(sleepFor)
        print "Reader %s: waking up" % self.getName()
        cu = cx.cursor()
        print "Reader %s: SELECTing" % self.getName()
        cu.execute("SELECT name, value FROM meta WHERE name='%s'" % self.getName())
        print "Reader %s: SELECTed %s" % (self.getName(), cu.fetchone())
        cu.close()
        cx.close()
        print "Reader %s: end" % self.getName()

def test_sqlite_busy():
    """Test handling of SQL_BUSY "errors" as discussed here:
        http://www.hwaci.com/sw/sqlite/faq.html#q7
        http://www.sqlite.org/cvstrac/wiki?p=MultiThreading
    
    Algorithm:
        - start one thread that will open the database and start modifying it
          then sleep for a while so other threads can get in there
        - have other thread(s) do selects from the database and see if they
          error out, if they block, if they timeout (play with timeout
          .connect() argument)
    """
    # Create a fresh starting database.
    if os.path.exists(dbname):
        os.remove(dbname)
    journal = dbname+"-journal"
    if os.path.exists(journal):
        os.remove(journal)
    cx = sqlite.connect(dbname)
    cu = cx.cursor()
    cu.execute("CREATE TABLE meta (name STRING, value STRING)")
    cx.commit()
    cu.close()
    cx.close()
    
    modifier = Modifier(dbname)
    readerNames = ("foo",) #XXX "bar", "baz")
    readers = [Reader(name, dbname) for name in readerNames]
    modifier.start()
    for reader in readers:
        reader.start()
    modifier.join()
    for reader in readers:
        reader.join()


if __name__ == "__main__":
    test_sqlite_busy()



#!/usr/bin/env python
import testsupport
import StringIO, unittest
import sqlite

class LogFileTemplate:
    def write(self, s):
        pass

class LogFile:
    def __init__(self):
        pass

def init_LogFile():
    LogFile.write = LogFileTemplate.write

class CommandLoggingTests(unittest.TestCase, testsupport.TestSupport):
    def tearDown(self):
        try:
            self.cnx.close()
            self.removefile()
        except AttributeError:
            pass
        except sqlite.InterfaceError:
            pass

    def CheckNoWrite(self):
        init_LogFile()
        del LogFile.write
        logger = LogFile()
        try:
            self.cnx = sqlite.connect(self.getfilename(),
                command_logfile=logger)

            self.fail("ValueError not raised")
        except ValueError:
            pass

    def CheckWriteNotCallable(self):
        logger = LogFile()
        logger.write = 5
        try:
            self.cnx = sqlite.connect(self.getfilename(),
                command_logfile=logger)

            self.fail("ValueError not raised")
        except ValueError:
            pass

    def CheckLoggingWorks(self):
        logger = StringIO.StringIO()

        expected_output = "\n".join([
            "BEGIN", "CREATE TABLE TEST(FOO INTEGER)",
            "INSERT INTO TEST(FOO) VALUES (5)",
            "ROLLBACK"]) + "\n"

        self.cnx = sqlite.connect(self.getfilename(),
            command_logfile=logger)
        cu = self.cnx.cursor()
        cu.execute("CREATE TABLE TEST(FOO INTEGER)")
        cu.execute("INSERT INTO TEST(FOO) VALUES (%i)", (5,))
        self.cnx.rollback()

        logger.seek(0)
        real_output = logger.read()
        
        if expected_output != real_output:
            self.fail("Logging didn't produce expected output.")

def suite():
    command_logging_suite = unittest.makeSuite(CommandLoggingTests, "Check")
    return command_logging_suite

def main():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
These are the tests for the low-level module _sqlite.

They try to execute as much of the low-level _sqlite module as possible to
facilitate coverage testing with the help of gcov.
"""

from __future__ import nested_scopes
import testsupport
import os, unittest, re
import _sqlite
from sqlite import ProgrammingError

class lowlevelTestCases(unittest.TestCase, testsupport.TestSupport):
    def setUp(self):
        self.filename = self.getfilename()
        self.cnx = _sqlite.connect(self.filename)

    def tearDown(self):
        try:
            self.cnx.close()
            self.removefile()
            del self.cnx
        except AttributeError:
            pass
        except ProgrammingError:
            pass

    def CheckModuleAttributeAccess(self):
        for attr in dir(_sqlite):
            _sqlite.__dict__[attr]

    def CheckConnectionAttributes(self):
        self.cnx.filename
        self.cnx.sql
        self.cnx.sqlite_changes()
        self.cnx.sqlite_last_insert_rowid()

        try:
            self.cnx.foo = 7
            self.fail("Could set attribute. Connection object should be read-only.")
        except TypeError:
            pass

    def CheckSQLiteExec(self):
        self.cnx.execute("create table test(id int, name varchar(20))")
        self.cnx.execute("insert into test(id, name) values (1, 'foo')")
        self.cnx.execute("insert into test(id, name) values (2, 'bar')")

        expected_colnames = ('id', 'name')
        expected_values = [('1', 'foo'), ('2', 'bar')]
        failures = []

        def callback(arg1, items, colnames):
            if colnames != expected_colnames:
                failures.append("expected colnames %s, got %s"
                                % (repr(expected_colnames), repr(colnames)))
            if items not in expected_values:
                failures.append("%s not in expected_values %s"
                                % (repr(items), repr(expected_values)))
            else:
                expected_values.pop(0)

        self.cnx.sqlite_exec("select * from test", callback, None)
        if len(failures) > 0:
            for failure in failures:
                self.fail(failure)

    def CheckSQLiteLastInsertRowID(self):
        self.cnx.execute("create table test(id integer primary key, name varchar(20))")
        self.cnx.execute("insert into test(id, name) values (NULL, 'foo')")
        self.cnx.execute("insert into test(id, name) values (NULL, 'bar')")
        rowid = self.cnx.sqlite_last_insert_rowid()
        self.failUnlessEqual(rowid, 2,
                            "last inserted rowid should have been %i, was %i"
                            % (2, rowid))

    def CheckSQLiteChanges(self):
        self.cnx.execute("create table test(id integer primary key, name varchar(20))")
        self.cnx.execute("insert into test(id, name) values (NULL, 'foo')")
        self.cnx.execute("insert into test(id, name) values (NULL, 'bar')")
        self.cnx.execute("insert into test(id, name) values (NULL, 'baz')")
        self.cnx.execute("delete from test where name='baz'")
        changed = self.cnx.sqlite_changes()
        self.failUnlessEqual(changed, 1,
                            "changed rows should have been %i, was %i"
                            % (1, changed))
        self.cnx.execute("update test set name='foobar' where id < 10")
        changed = self.cnx.sqlite_changes()
        self.failUnlessEqual(changed, 2,
                            "changed rows should have been %i, was %i"
                            % (2, changed))

    def CheckConnectionForProgrammingError(self):
        self.cnx.close()
        self.removefile()

        self.failUnlessRaises(ProgrammingError, self.cnx.close)
        self.failUnlessRaises(ProgrammingError, self.cnx.execute, "")

    def CheckConnectionForNumberOfArguments(self):
        self.failUnlessRaises(TypeError, self.cnx.close, None)
        self.failUnlessRaises(TypeError, self.cnx.execute, None, None)
        self.failUnlessRaises(TypeError, self.cnx.sqlite_changes, None)
        self.failUnlessRaises(TypeError, self.cnx.sqlite_exec, None)
        self.failUnlessRaises(TypeError, self.cnx.sqlite_last_insert_rowid, None)

    def CheckConnectionDestructor(self):
        del self.cnx
        self.removefile()

    def CheckResultObject(self):
        create_statement = "create table test(id INTEGER, name TEXT)"
        self.cnx.execute(create_statement)

        self.failUnlessEqual(create_statement, self.cnx.sql,
            ".sql should have been %s, was %s" % (create_statement, self.cnx.sql))

        self.cnx.execute("insert into test(id, name) values (4, 'foo')")
        self.cnx.execute("insert into test(id, name) values (5, 'bar')")

        res = self.cnx.execute("select id, name from test")
        self.failUnless(res.rowcount == 2, "Should have returned 2 rows, but was %i" % res.rowcount)

        correct_col_defs = (('id', _sqlite.INTEGER, None, None, None, None, None), \
                            ('name', _sqlite.STRING, None, None, None, None, None))
        self.assertEqual(res.col_defs, correct_col_defs,
            "col_defs should have been %s, was %s" % (repr(correct_col_defs), repr(res.col_defs)))

        correct_row_list = [(4, 'foo'), (5, 'bar')]
        self.assertEqual(res.row_list, correct_row_list,
            "rowlist should have been %s, was %s" % (repr(correct_row_list), repr(res.row_list)))

    def CheckResultAttributes(self):
        res = self.cnx.execute("select NULL, max(4,5)")
        try:
            res.foo = 7

        except TypeError:
            pass

    def CheckSQLiteVersion(self):
        try:
            ver = _sqlite.sqlite_version()
        except:
            self.fail('sqlite_version() failed')
        pat = re.compile(r'\d*\.\d*\.\d*')
        if not re.match(pat,ver):
            self.fail('Incorrect sqlite_version() format, '
                'should be digits.digits.digits, was %s'%ver)


def suite():
    return unittest.makeSuite(lowlevelTestCases, "Check")

def main():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    main()



#!/usr/bin/env python
import testsupport
import os, string, sys, types, unittest
import sqlite

class TransactionTests(unittest.TestCase, testsupport.TestSupport):
    def setUp(self):
        self.filename = self.getfilename()
        self.cnx = sqlite.connect(self.filename)
        self.cur = self.cnx.cursor()

    def tearDown(self):
        try:
            self.cnx.close()
            self.removefile()
        except AttributeError:
            pass
        except sqlite.InterfaceError:
            pass

    def CheckValueInTransaction(self):
        self.cur.execute("create table test (a)")
        self.cur.execute("insert into test (a) values (%s)", "foo")
        self.cur.execute("-- types int")
        self.cur.execute("select count(a) as count from test")
        res = self.cur.fetchone()
        self.failUnlessEqual(res.count, 1,
                             "Wrong number of rows during transaction.")

    def CheckValueAfterCommit(self):
        self.cur.execute("create table test (a)")
        self.cur.execute("insert into test (a) values (%s)", "foo")
        self.cur.execute("-- types int")
        self.cur.execute("select count(a) as count from test")
        self.cnx.commit()
        res = self.cur.fetchone()
        self.failUnlessEqual(res.count, 1,
                             "Wrong number of rows during transaction.")

    def CheckValueAfterRollback(self):
        self.cur.execute("create table test (a)")
        self.cnx.commit()
        self.cur.execute("insert into test (a) values (%s)", "foo")
        self.cnx.rollback()
        self.cur.execute("-- types int")
        self.cur.execute("select count(a) as count from test")
        res = self.cur.fetchone()
        self.failUnlessEqual(res.count, 0,
                             "Wrong number of rows during transaction.")

    def CheckImmediateCommit(self):
        try:
            self.cnx.commit()
        except:
            self.fail("Immediate commit raises exeption.")

    def CheckImmediateRollback(self):
        try:
            self.cnx.rollback()
        except:
            self.fail("Immediate rollback raises exeption.")

class AutocommitTests(unittest.TestCase, testsupport.TestSupport):
    def setUp(self):
        self.filename = self.getfilename()
        self.cnx = sqlite.connect(self.filename, autocommit=1)
        self.cur = self.cnx.cursor()

    def tearDown(self):
        try:
            self.cnx.close()
            self.removefile()
        except AttributeError:
            pass
        except sqlite.InterfaceError:
            pass

    def CheckCommit(self):
        self.cur.execute("select abs(5)")
        try:
            self.cnx.commit()
        except:
            self.fail(".commit() raised an exception")

    def CheckRollback(self):
        self.cur.execute("select abs(5)")
        self.failUnlessRaises(sqlite.ProgrammingError, self.cnx.rollback)

class ChangeAutocommitTests(unittest.TestCase):
    pass

def suite():
    transaction_tests = unittest.makeSuite(TransactionTests, "Check")
    autocommit_tests = unittest.makeSuite(AutocommitTests, "Check")
    change_autocommit_tests = unittest.makeSuite(ChangeAutocommitTests, "Check")

    test_suite = unittest.TestSuite((transaction_tests, autocommit_tests,
                                    change_autocommit_tests))
    return test_suite

def main():
    runner = unittest.TextTestRunner()
    runner.run(suite())

if __name__ == "__main__":
    main()



#!/usr/bin/env python
import _sqlite
import os, tempfile, unittest

class TestSupport:
    def getfilename(self):
        if _sqlite.sqlite_version_info() >= (2, 8, 2):
            return ":memory:"
        else:
            return tempfile.mktemp()

    def removefile(self):
        if self.filename != ":memory:":
            os.remove(self.filename)



import os
import sqlite

# Ok, let's define a user-defined type we can use with the SQLite database
class Point:
    def __init__(self, x, y):
        self.x, self.y = x, y

    # The _quote function is currently the way a PySQLite user-defined type
    # returns its string representation to write to the database.
    def _quote(self):
        return "'%f,%f'" % (self.x, self.y)

    def __str__(self):
        return "Point(%f, %f)" % (self.x, self.y)

# The conversion callable needs to accept a string, parse it and return an
# instance of your user-defined type.
def pointConverter(s):
    x, y = s.split(",")
    return Point(float(x), float(y))

# Ensure we have an empty database
if os.path.exists("db"): os.remove("db")

cx = sqlite.connect("db", converters={"point": pointConverter}) 
cu = cx.cursor()
cu.execute("create table test(p point, n int)")
cu.execute("insert into test(p, n) values (%s, %s)", (Point(-3.2, 4.5), 25))

# For user-defined types, and for statements which return anything but direct
# columns, you need to use the "-- types" feature of PySQLite:
cu.execute("-- types point, int")
cu.execute("select p, n from test")
row = cu.fetchone()

print "p:", row.p       # .columnname instead of [0] is a PySQLite
print "n:", row.n       # extension to the DB-API! 
cx.close()



#!/usr/bin/env python

import os, sys
from distutils.core import setup
from distutils.extension import Extension

__version__ = "1.0.1"

sqlite = "sqlite"
sources = ["_sqlite.c", "encode.c", "port/strsep.c"]
macros = []

if sys.platform in ("linux-i386", "linux2"): # most Linux
    include_dirs = ['/usr/include/sqlite']
    library_dirs = ['/usr/lib/']
    libraries = [sqlite]
    runtime_library_dirs = []
    extra_objects = []
elif sys.platform in ("freebsd4", "freebsd5", "openbsd2", "cygwin", "darwin"):
    if sys.platform == "darwin":
        LOCALBASE = os.environ.get("LOCALBASE", "/opt/local")
    else:
        LOCALBASE = os.environ.get("LOCALBASE", "/usr/local")
    include_dirs = ['%s/include' % LOCALBASE]
    library_dirs = ['%s/lib/' % LOCALBASE]
    libraries = [sqlite]
    runtime_library_dirs = []
    extra_objects = []
elif sys.platform == "win32":
    include_dirs = [r'..\sqlite']
    library_dirs = [r'..\sqlite']
    libraries = ["sqlite"]
    runtime_library_dirs = []
    extra_objects = []
elif os.name == "posix": # most Unixish platforms
    include_dirs = ['/usr/local/include']
    library_dirs = ['/usr/local/lib']
    libraries = [sqlite]
    # On some platorms, this can be used to find the shared libraries
    # at runtime, if they are in a non-standard location. Doesn't
    # work for Linux gcc.
    ## runtime_library_dirs = library_dirs
    runtime_library_dirs = []
    # This can be used on Linux to force use of static sqlite lib
    ## extra_objects = ['/usr/lib/sqlite/libsqlite.a']
    extra_objects = []
else:
    raise "UnknownPlatform", "sys.platform=%s, os.name=%s" % \
          (sys.platform, os.name)
    
long_description = \
"""Python interface to SQLite

pysqlite is an interface to the SQLite database server for Python. It aims to be
fully compliant with Python database API version 2.0 while also exploiting the
unique features of SQLite.

"""

def main():
    py_modules = ["sqlite.main"]
                          
    # patch distutils if it can't cope with the "classifiers" keyword
    if sys.version < '2.2.3':
        from distutils.dist import DistributionMetadata
        DistributionMetadata.classifiers = None
        DistributionMetadata.download_url = None
    
    setup ( # Distribution meta-data
            name = "pysqlite",
            version = __version__,
            description = "An interface to SQLite",
            long_description=long_description,
            author = "PySQLite developers",
            author_email = "pysqlite-devel@lists.sourceforge.net",
            license = "Python license",
            platforms = "ALL",
            url = "http://pysqlite.sourceforge.net/",

            # Description of the modules and packages in the distribution
            py_modules = py_modules,

            ext_modules = [Extension( name='_sqlite',
                                      sources=sources,
                                      include_dirs=include_dirs,
                                      library_dirs=library_dirs,
                                      runtime_library_dirs=runtime_library_dirs,
                                      libraries=libraries,
                                      extra_objects=extra_objects,
                                      define_macros=macros
                                      )],
            classifiers = [
            "Development Status :: 5 - Production/Stable",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Operating System :: Microsoft :: Windows :: Windows NT/2000",
            "Operating System :: POSIX",
            "Programming Language :: C",
            "Programming Language :: Python",
            "Topic :: Database :: Database Engines/Servers",
            "Topic :: Database :: Front-Ends"]
            )

if __name__ == "__main__":
    main()



import sys
import sqlite

# The shared connection object
cx = None

def getCon():
    # All code gets the connection object via this function
    global cx
    return cx

def createSchema():
    # Create the schema and make sure we're not accessing an old, incompatible schema
    cu = getCon().cursor()
    cu.execute("select tbl_name from sqlite_master where type='table' order by tbl_name")
    tables = []
    for row in cu.fetchall():
        tables.append(row.tbl_name)
    if tables != ["customer", "orders"]:
        if tables == []:
            # ok, database is empty
            cu.execute("begin")
            cu.execute("""
                create table customer (
                    cust_id integer primary key,
                    cust_firstname text not null,
                    cust_lastname text not null,
                    cust_no text not null
                )
                """)
            cu.execute("""
                create table orders (
                    ord_id integer primary key,
                    ord_customer int,
                    ord_item text not null,
                    ord_quantity integer
                )
            """)
            cu.execute("commit")
        else:
            print "We have an unknown schema here. Please fix manually."
            sys.exit(1)

def createCustomer(firstname, lastname, customerNo):
    # Create a new customer and return the primary key id.
    cu = getCon().cursor()
    cu.execute("""
        insert into customer(cust_firstname, cust_lastname, cust_no)
            values (%s, %s, %s)
            """, (firstname, lastname, customerNo))
    return cu.lastrowid

def createOrder(cust_id, ord_item, ord_quantity):
    # Create a new order for the customer identified by cust_id and return the
    # primary key of the created order row.
    cu = getCon().cursor()
    cu.execute("""
        insert into orders (ord_customer, ord_item, ord_quantity)
            values (%s, %s, %s)
            """, (cust_id, ord_item, ord_quantity))
    return cu.lastrowid

def deleteOrder(ord_id):
    # Delete an order.
    cu = getCon().cursor()
    cu.execute("delete from order where ord_id=%s", (ord_id,))

def deleteCustomer(cust_id):
    # Delete the customer identified by cust_id and all its orders (recursive
    # delete).

    # So now, finally, here we have an example where you *really* need
    # transactions. We either want this to happen all or not at all. So all of
    # these SQL statements need to be atomic, i. e. we need a transaction here.
    cu = getCon().cursor()

    cu.execute("begin")
    cu.execute("delete from orders where ord_customer=%s", (cust_id,))
    cu.execute("delete from customer where cust_id=%s", (cust_id,))
    cu.execute("commit")

def main():
    global cx
    # Open the connection in autocommit mode, because we believe we have reason
    # to :-/
    cx = sqlite.connect("customerdb", autocommit=1)
    createSchema()

    # Create a customer
    cust_id = createCustomer("Jane", "Doe", "JD0001")

    # Create two orders for the customer
    ord_id = createOrder(cust_id, "White Towel", 2)
    ord_id = createOrder(cust_id, "Blue Cup", 5)

    # Delete the customer, and all her orders.
    deleteCustomer(cust_id)

    cx.close()

if __name__ == "__main__":
    main()
